/*
 * Distinguisher for LUX reduced to 8 blank rounds in the output step.
 * Author: Tor E. Bj{\o}rstad, February 2009
 * Based on idea by Peter Schmidt-Nielsen
 */

/*
 * To compile: "gcc -o test test.c"
 */

#include <stdio.h>
#include "LUX.c"

/*
 * LUX.c is taken from the Reference Implementation, except for
 * one line which is changed to reduce the number of blank rounds
 * after padding.
 * 
 * Change the following:
 * Previous line 283:	for(i=0;i<16;i++){
 * New      line 283:	for(i=0;i< 8;i++){
 */

int main ()
{
	int i;

	/* 4 input blocks */
	BitSequence zero [4] = {0x00, 0x00, 0x00, 0x00}; // The input blocks can be anything. All-zeros for simplicity only.
	BitSequence foo  [4] = {0x00, 0x00, 0x00, 0x00};
	BitSequence bar  [4] = {0x00, 0x00, 0x00, 0x00};
	BitSequence baz  [4] = {0x00, 0x00, 0x00, 0x00};

	/* 2 buffers for storing hash outputs */
	BitSequence out [32];
	BitSequence xor [32];

	/* 2 hash function states to play with */
	hashState ctx;

	/* Set the xor buffer to 0 */
	memset (xor, 0x0, 32);

	/* Compute the hash of 2 all-zero blocks. */
	Init (&ctx, 256);
	Update (&ctx, zero, 32);

	/* Also store the leftmost column of the core after one block. */
	baz [0] = ctx.cr [3] [0]; baz [1] = ctx.cr [2] [0]; baz [2] = ctx.cr [1] [0]; baz [3] = ctx.cr [0] [0];
	Update (&ctx, zero, 32);
	Final (&ctx, xor);

	/*
	 * Now we cycle through the other 255 possible values of the top
	 * byte in the first input word, and fix the differential by
	 * adding the necessary difference in the second input word.
	 */

	for (foo [3] = 0x01; foo [3] != 0; ++foo [3])
	{
		/* Initialise the hash state */
		Init (&ctx, 256);

		/* Insert a difference in ctx [1] */
		Update (&ctx, foo, 32);

		/* Compute the 2nd difference block */
		bar [0] = ctx.cr [3] [0] ^ baz [0];
		bar [1] = ctx.cr [2] [0] ^ baz [1];
		bar [2] = ctx.cr [1] [0] ^ baz [2];
		bar [3] = ctx.cr [0] [0] ^ baz [3];

		/* Cancel the difference in the core */
		Update (&ctx, bar, 32);

		/* Compute the hash and XOR the result into the buffer */
		Final (&ctx, out);
		for (i = 0; i < 32; ++i)
		{
			xor [i] ^= out [i];
		}
	}

	/*
	 * Print the XOR of the 256 hashed values. Note that the first byte is 00 00 00 00.
	 */

	printf ("XOR of 256 hashes is:\n");
	for (i = 0; i < 32; ++i)
		printf ("%02x ", xor [i]); printf ("\n");

	return 0;
}
